<!DOCTYPE html>
<html>
    <head>
        <meta charset="utf-8">
        <meta http-equiv="refresh" content="{{ redirect_in.total_seconds }};url={{ redirect_url }}" />
        <title>Redirect</title>
    </head>
    <body>
        Please wait, we'll redirect you shortly...
        {% include "fingerprint/include/script.html" %}
    </body>
</html>