# Generated by Django 4.2.20 on 2025-03-11 19:52

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("fingerprint", "0004_requestfingerprint_referer_country"),
    ]

    operations = [
        migrations.AddField(
            model_name="usersession",
            name="utm_campaign",
            field=models.CharField(blank=True, max_length=255),
        ),
        migrations.AddField(
            model_name="usersession",
            name="utm_content",
            field=models.CharField(blank=True, max_length=255),
        ),
        migrations.AddField(
            model_name="usersession",
            name="utm_medium",
            field=models.CharField(blank=True, max_length=255),
        ),
        migrations.AddField(
            model_name="usersession",
            name="utm_source",
            field=models.CharField(blank=True, max_length=255),
        ),
        migrations.AddField(
            model_name="usersession",
            name="utm_term",
            field=models.CharField(blank=True, max_length=255),
        ),
    ]
