# Generated by Django 4.2.4 on 2023-08-10 14:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("fingerprint", "0001_initial"),
    ]

    operations = [
        migrations.CreateModel(
            name="Url",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("value", models.CharField(max_length=255, unique=True)),
            ],
        ),
        migrations.AddField(
            model_name="browserfingerprint",
            name="url",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="%(model_name)ss",
                to="fingerprint.url",
            ),
        ),
        migrations.AddField(
            model_name="requestfingerprint",
            name="url",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="%(model_name)ss",
                to="fingerprint.url",
            ),
        ),
        migrations.AddIndex(
            model_name="browserfingerprint",
            index=models.Index(fields=["url", "user_session"], name="fingerprint_url_id_9faee4_idx"),
        ),
        migrations.AddIndex(
            model_name="requestfingerprint",
            index=models.Index(fields=["url", "user_session"], name="fingerprint_url_id_6c4d95_idx"),
        ),
    ]
