# Generated by Django 6.0.2 on 2026-02-05 10:39

from django.db import migrations

import fingerprint.fields


class Migration(migrations.Migration):
    dependencies = [
        ("fingerprint", "0006_usersession_referer"),
    ]

    operations = [
        migrations.AlterField(
            model_name="url",
            name="value",
            field=fingerprint.fields.TruncatedCharField(max_length=2048, unique=True),
        ),
    ]
